<?php

/*
 * This file is part of EC-CUBE
 *
 * Copyright(c) LOCKON CO.,LTD. All Rights Reserved.
 *
 * http://www.lockon.co.jp/
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Plugin\ForIbn4204\Controller;

use Eccube\Controller\AbstractController;
use Plugin\ForIbn4204\Entity\Categoryimage;
use Eccube\Event\EccubeEvents;
use Eccube\Event\EventArgs;
use Plugin\ForIbn4204\Form\Type\CategoryimageType;
use Plugin\ForIbn4204\Repository\CategoryimageRepository;
use Eccube\Util\CacheUtil;
use Knp\Component\Pager\Paginator;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\Routing\Annotation\Route;

//▼画像アップロード
use Symfony\Component\Filesystem\Filesystem;
use Symfony\Component\HttpKernel\Exception\UnsupportedMediaTypeHttpException;
use Symfony\Component\HttpFoundation\Response;

class CategoryimageController extends AbstractController
{
    /**
     * @var CategoryimageRepository
     */
    protected $categoryimageRepository;

    /**
     * CategoryimageController constructor.
     *
     * @param CategoryimageRepository $categoryimageRepository
     */
    public function __construct(CategoryimageRepository $categoryimageRepository)
    {
        $this->categoryimageRepository = $categoryimageRepository;
    }

    /**
     * 登録・編集する。
     *
     * @Route("/%eccube_admin_route%/plugin/categoryimage", name="plugin_categoryimage")
     * @Route("/%eccube_admin_route%/plugin/categoryimage/{category_id}/edit", requirements={"category_id" = "\d+"}, name="plugin_categoryimage_edit")
     * @Template("@ForIbn4204/admin/categoryimage_edit.twig")
     *
     * @param Request $request
     * @param null $id
     *
     * @return array|\Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function edit(Request $request, Categoryimage $Categoryimage = null, CacheUtil $cacheUtil)
    {
        if (is_null($Categoryimage)) {
            $Categoryimage = $this->categoryimageRepository->findOneBy([], ['sort_no' => 'DESC']);
            $sortNo = 1;
            if ($Categoryimage) {
                $sortNo = $Categoryimage->getSortNo() + 1;
            }

            $Categoryimage = new \Plugin\ForIbn4204\Entity\Categoryimage();
            $Categoryimage
                ->setSortNo($sortNo)
                ->setFixed(true)
                ->setVisible(true);
        }

        $builder = $this->formFactory
            ->createBuilder(CategoryimageType::class, $Categoryimage);

        $form = $builder->getForm();
		
        //▼画像アップロード
		// 既に画像保存されてる場合は取得する
        $oldCategoryimageImage = $Categoryimage->getCategoryimageImage();
        $form->setData($Categoryimage);
		//▲

        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {

			//▼画像アップロード
            $Categoryimage = $form->getData();
            // ファイルアップロード
            $file = $form['categoryimage_image']->getData();
            $fs = new Filesystem();
            if ($file && $fs->exists($this->getParameter('eccube_temp_image_dir').'/'.$file)) {
                $fs->rename(
                    $this->getParameter('eccube_temp_image_dir').'/'.$file,
                    $this->getParameter('eccube_save_image_dir').'/'.$file
                );
            }
			//▲
			
            $this->categoryimageRepository->save($Categoryimage);

            $this->addSuccess('admin.common.save_complete', 'admin');

            // キャッシュの削除
            $cacheUtil->clearDoctrineCache();

            return $this->redirectToRoute('admin_product_category');
        }

        return [
            'form' => $form->createView(),
            'Categoryimage' => $Categoryimage,
            'oldCategoryimageImage' => $oldCategoryimageImage, //▼画像アップロード
        ];
    }

    //▼画像アップロード
	/**
     * @Route("/%eccube_admin_route%/plugin/categoryimage/image/process", name="admin_categoryimage_image_process")
     */
    public function imageProcess(Request $request)
    {
        if (!$request->isXmlHttpRequest()) {
            throw new BadRequestHttpException();
        }

        $images = $request->files->get('admin_categoryimage'); //CategoryimageType.phpの最下部、return 'admin_categoryimage'
        $allowExtensions = ['gif', 'jpg', 'jpeg', 'png'];
        $filename = null;
        if (isset($images['categoryimage_image_file'])) {
            $image = $images['categoryimage_image_file'];

            //ファイルフォーマット検証
            $mimeType = $image->getMimeType();
            if (0 !== strpos($mimeType, 'image')) {
                throw new UnsupportedMediaTypeHttpException();
            }

            // 拡張子
            $extension = $image->getClientOriginalExtension();
            if (!in_array($extension, $allowExtensions)) {
                throw new UnsupportedMediaTypeHttpException();
            }

            $filename = date('mdHis').uniqid('_').'.'.$extension;
            $image->move($this->getParameter('eccube_temp_image_dir'), $filename);
        }

        return new Response($filename);
    }

	// ▼画像アップロード
    /**
     * アップロード画像を取得する際にコールされるメソッド.
     *
     * @see https://pqina.nl/filepond/docs/api/server/#load
     * @Route("/%eccube_admin_route%/plugin/categoryimage/image/load", name="admin_categoryimage_image_load")
     */
    public function imageLoad(Request $request)
    {
        if (!$request->isXmlHttpRequest()) {
            throw new BadRequestHttpException();
        }

        $dirs = [
            $this->eccubeConfig['eccube_save_image_dir'],
            $this->eccubeConfig['eccube_temp_image_dir'],
        ];

        foreach ($dirs as $dir) {
            $image = \realpath($dir.'/'.$request->query->get('source'));
            $dir = \realpath($dir);

            if (\is_file($image) && \str_starts_with($image, $dir)) {
                $file = new \SplFileObject($image);

                return $this->file($file, $file->getBasename());
            }
        }

        throw new NotFoundHttpException();
    }

	// ▼画像アップロード
    /**
     * アップロード画像をすぐ削除する際にコールされるメソッド.
     *
     * @see https://pqina.nl/filepond/docs/api/server/#revert
     * @Route("/%eccube_admin_route%/plugin/categoryimage/image/revert", name="admin_categoryimage_image_revert")
     */
    public function imageRevert(Request $request)
    {
        if (!$request->isXmlHttpRequest() && $this->isTokenValid()) {
            throw new BadRequestHttpException();
        }

        $tempFile = $this->eccubeConfig['eccube_temp_image_dir'].'/'.$request->getContent();
        if (is_file($tempFile) && stripos(realpath($tempFile), $this->eccubeConfig['eccube_temp_image_dir']) === 0) {
            $fs = new Filesystem();
            $fs->remove($tempFile);

            return new Response(null, Response::HTTP_NO_CONTENT);
        }

        throw new NotFoundHttpException();
    }

    /**
     * 指定したデータを削除する。
     *
     * @Route("/%eccube_admin_route%/plugin/categoryimage/{id}/delete", requirements={"id" = "\d+"}, name="plugin_categoryimage_delete", methods={"DELETE"})
     *
     * @param Request $request
     * @param Categoryimage $Categoryimage
     *
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function delete(Request $request, Categoryimage $Categoryimage, CacheUtil $cacheUtil)
    {
        $this->isTokenValid();

        log_info('バナー画像・カテゴリ説明削除開始', [$Categoryimage->getId()]);

        try {
            $this->categoryimageRepository->delete($Categoryimage);

            $this->addSuccess('admin.common.delete_complete', 'admin');

            log_info('バナー画像・カテゴリ説明削除完了', [$Categoryimage->getId()]);

            // キャッシュの削除
            $cacheUtil->clearDoctrineCache();

        } catch (\Exception $e) {
            $message = trans('admin.common.delete_error_foreign_key', ['%name%' => $Categoryimage->getCategoryImage()]);
            $this->addError($message, 'admin');

            log_error('バナー画像・カテゴリ説明削除エラー', [$Categoryimage->getId(), $e]);
        }

        $sortNo = 1;
        $Categoryimages = $this->categoryimageRepository
            ->findBy([], ['sort_no' => 'ASC']);

        foreach ($Categoryimages as $Categoryimage) {
                $Categoryimage->setSortNo($sortNo);
                $sortNo++;
        }
        $this->entityManager->flush();

        return $this->redirectToRoute('admin_product_category');
    }

    /**
     * @Route("/%eccube_admin_route%/plugin/categoryimage/{id}/visibility", requirements={"id" = "\d+"}, name="plugin_categoryimage_visibility", methods={"PUT"})
     */
    public function visibility(Request $request, Categoryimage $Categoryimage)
    {
        $this->isTokenValid();

        // 表示・非表示を切り替える
        if ($Categoryimage->isVisible()) {
            $message = trans('admin.common.to_hide_complete', ['%name%' => $Categoryimage->getId()]);
            $Categoryimage->setVisible(false);
        } else {
            $message = trans('admin.common.to_show_complete', ['%name%' => $Categoryimage->getId()]);
            $Categoryimage->setVisible(true);
        }
        $this->entityManager->persist($Categoryimage);

        $this->entityManager->flush();

        $this->addSuccess($message, 'admin');

        return $this->redirectToRoute('admin_product_categoryimage');
    }

    /**
     * @Route("/%eccube_admin_route%/plugin/categoryimage/sort_no/move", name="plugin_categoryimage_sort_no_move", methods={"POST"})
     *
     * @param Request $request
     *
     * @return Response
     */
    public function moveSortNo(Request $request)
    {
        if (!$request->isXmlHttpRequest()) {
            throw new BadRequestHttpException();
        }

        if ($this->isTokenValid()) {
            $sortNos = $request->request->all();
            foreach ($sortNos as $categoryimageId => $sortNo) {
                $Categoryimage = $this->categoryimageRepository->find($categoryimageId);
                $Categoryimage->setSortNo($sortNo);
                $this->entityManager->persist($Categoryimage);
            }
            $this->entityManager->flush();
        }

        return $this->json('OK', 200);
    }

}
