<?php

/*
 * This file is part of EC-CUBE
 *
 * Copyright(c) LOCKON CO.,LTD. All Rights Reserved.
 *
 * http://www.lockon.co.jp/
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Plugin\ForIbn4204\Controller;

use Eccube\Controller\AbstractController;
use Plugin\ForIbn4204\Entity\Ibn4204Carousel;
use Plugin\ForIbn4204\Form\Type\Ibn4204CarouselType;
use Plugin\ForIbn4204\Repository\Ibn4204CarouselRepository;
use Eccube\Util\CacheUtil;
use Knp\Component\Pager\PaginatorInterface;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use Symfony\Component\Form\Form;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;


/**
 * Class Ibn4204CarouselController.
 */
class Ibn4204CarouselController extends AbstractController
{
    /**
     * @var Ibn4204CarouselRepository
     */
    private $ibn4204CarouselRepository;

    /**
     * Ibn4204CarouselController constructor.
     *
     * @param Ibn4204CarouselRepository $ibn4204CarouselRepository
     */
    public function __construct(Ibn4204CarouselRepository $ibn4204CarouselRepository)
    {
        $this->ibn4204CarouselRepository = $ibn4204CarouselRepository;
    }

    /**
     * 一覧を表示する。
     *
     * @param Request     $request
     *
     * @return array
     * @Route("/%eccube_admin_route%/content/ibn4204_carousel", name="admin_content_ibn4204_carousel")
     * @Route("/%eccube_admin_route%/content/ibn4204_carousel/page/{page_no}", requirements={"page_no" = "\d+"}, name="admin_content_ibn4204_carousel_page")
     * @Template("@ForIbn4204/admin/ibn4204_carousel.twig")
     */
    public function index(Request $request, PaginatorInterface $paginator, $page_no = 1)
    {
        $qb = $this->ibn4204CarouselRepository
            ->findBy(
                [],
                ['sort_no' => 'DESC']
            );

        $pagination = $paginator->paginate(
            $qb,
            $page_no,
            $this->eccubeConfig->get('eccube_default_page_count')
        );

        return [
            'pagination' => $pagination,
        ];
    }

    /**
     * 登録・編集する。
     *
     * @Route("/%eccube_admin_route%/content/ibn4204_carousel/new", name="admin_content_ibn4204_carousel_new")
     * @Route("/%eccube_admin_route%/content/ibn4204_carousel/{id}/edit", requirements={"id" = "\d+"}, name="admin_content_ibn4204_carousel_edit")
     * @Template("@ForIbn4204/admin/ibn4204_carousel_edit.twig")
     */
    public function edit(Request $request, Ibn4204Carousel $Ibn4204Carousel = null, CacheUtil $cacheUtil)
    {
        if (is_null($Ibn4204Carousel)) {
            $Ibn4204Carousel = $this->ibn4204CarouselRepository->findOneBy([], ['sort_no' => 'DESC']);
            $sortNo = 1;
            if ($Ibn4204Carousel) {
                $sortNo = $Ibn4204Carousel->getSortNo() + 1;
            }

            $Ibn4204Carousel = new \Plugin\ForIbn4204\Entity\Ibn4204Carousel();
            $Ibn4204Carousel
                ->setSortNo($sortNo)
                ->setFixed(true)
                ->setVisible(true);
        }

        $builder = $this->formFactory
            ->createBuilder(Ibn4204CarouselType::class, $Ibn4204Carousel);

        $form = $builder->getForm();
		
        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {

            $this->ibn4204CarouselRepository->save($Ibn4204Carousel);

            $this->addSuccess('admin.common.save_complete', 'admin');

            // キャッシュの削除
            $cacheUtil->clearDoctrineCache();

            return $this->redirectToRoute('admin_content_ibn4204_carousel_edit', ['id' => $Ibn4204Carousel->getId()]);
        }

        return [
            'form' => $form->createView(),
            'Ibn4204Carousel' => $Ibn4204Carousel,
        ];
    }

    /**
     * 指定したデータを削除する。
     *
     * @param Request     $request
     * @param Ibn4204Carousel $Ibn4204Carousel
     *
     * @throws \Exception
     *
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     * @Route("/%eccube_admin_route%/content/ibn4204_carousel/{id}/delete", requirements={"id" = "\d+"}, name="admin_content_ibn4204_carousel_delete", methods={"DELETE"})
     */
    public function delete(Request $request, Ibn4204Carousel $Ibn4204Carousel, CacheUtil $cacheUtil)
    {
        $this->isTokenValid();

        log_info('ティッカー削除開始', [$Ibn4204Carousel->getId()]);

        try {
            $this->ibn4204CarouselRepository->delete($Ibn4204Carousel);

            $this->addSuccess('admin.common.delete_complete', 'admin');

            log_info('ティッカー削除完了', [$Ibn4204Carousel->getId()]);

            // キャッシュの削除
            $cacheUtil->clearDoctrineCache();

        } catch (\Exception $e) {
            $message = trans('admin.common.delete_error_foreign_key', ['%name%' => $Ibn4204Carousel->getTopImage()]);
            $this->addError($message, 'admin');

            log_error('ティッカー削除エラー', [$Ibn4204Carousel->getId(), $e]);
        }

        $sortNo = 1;
        $Ibn4204Carousels = $this->ibn4204CarouselRepository
            ->findBy([], ['sort_no' => 'ASC']);

        foreach ($Ibn4204Carousels as $Ibn4204Carousel) {
                $Ibn4204Carousel->setSortNo($sortNo);
                $sortNo++;
        }
        $this->entityManager->flush();

        return $this->redirectToRoute('admin_content_ibn4204_carousel');
    }

    /**
     * @Route("/%eccube_admin_route%/content/ibn4204_carousel/{id}/visibility", requirements={"id" = "\d+"}, name="admin_content_ibn4204_carousel_visibility", methods={"PUT"})
     */
    public function visibility(Request $request, Ibn4204Carousel $Ibn4204Carousel)
    {
        $this->isTokenValid();

        // 表示・非表示を切り替える
        if ($Ibn4204Carousel->isVisible()) {
            $message = trans('admin.common.to_hide_complete', ['%name%' => $Ibn4204Carousel->getId()]);
            $Ibn4204Carousel->setVisible(false);
        } else {
            $message = trans('admin.common.to_show_complete', ['%name%' => $Ibn4204Carousel->getId()]);
            $Ibn4204Carousel->setVisible(true);
        }
        $this->entityManager->persist($Ibn4204Carousel);

        $this->entityManager->flush();

        $this->addSuccess($message, 'admin');

        return $this->redirectToRoute('admin_content_ibn4204_carousel');
    }

    /**
     * @Route("/%eccube_admin_route%/content/ibn4204_carousel/sort_no/move", name="admin_content_ibn4204_carousel_sort_no_move", methods={"POST"})
     *
     * @param Request $request
     *
     * @return Response
     */
    public function moveSortNo(Request $request)
    {
        if (!$request->isXmlHttpRequest()) {
            throw new BadRequestHttpException();
        }

        if ($this->isTokenValid()) {
            $sortNos = $request->request->all();
            foreach ($sortNos as $ibn4204CarouselId => $sortNo) {
                $Ibn4204Carousel = $this->ibn4204CarouselRepository->find($ibn4204CarouselId);
                $Ibn4204Carousel->setSortNo($sortNo);
                $this->entityManager->persist($Ibn4204Carousel);
            }
            $this->entityManager->flush();
        }

        return $this->json('OK', 200);
    }
}
