<?php

/*
 * This file is part of EC-CUBE
 *
 * Copyright(c) LOCKON CO.,LTD. All Rights Reserved.
 *
 * http://www.lockon.co.jp/
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Plugin\ForIbn4204\Controller;

use Eccube\Controller\AbstractController;
use Plugin\ForIbn4204\Entity\Ibn4204Topic;
use Eccube\Event\EccubeEvents;
use Eccube\Event\EventArgs;
use Plugin\ForIbn4204\Form\Type\Ibn4204TopicType;
use Plugin\ForIbn4204\Repository\Ibn4204TopicRepository;
use Eccube\Util\CacheUtil;
use Knp\Component\Pager\PaginatorInterface;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\Routing\Annotation\Route;

//▼画像アップロード
use Symfony\Component\Filesystem\Filesystem;
use Symfony\Component\HttpKernel\Exception\UnsupportedMediaTypeHttpException;
use Symfony\Component\HttpFoundation\Response;

class Ibn4204TopicController extends AbstractController
{
    /**
     * @var Ibn4204TopicRepository
     */
    protected $ibn4204TopicRepository;

    /**
     * Ibn4204TopicController constructor.
     *
     * @param Ibn4204TopicRepository $ibn4204TopicRepository
     */
    public function __construct(Ibn4204TopicRepository $ibn4204TopicRepository)
    {
        $this->ibn4204TopicRepository = $ibn4204TopicRepository;
    }

    /**
     * 一覧を表示する。
     *
     * @Route("/%eccube_admin_route%/content/ibn4204_topic", name="admin_content_ibn4204_topic")
     * @Route("/%eccube_admin_route%/content/ibn4204_topic/page/{page_no}", requirements={"page_no" = "\d+"}, name="admin_content_ibn4204_topic_page")
     * @Template("@ForIbn4204/admin/ibn4204_topic.twig")
     *
     * @param Request $request
     * @param int $page_no
     * @param Paginator $paginator
     *
     * @return array
     */
    public function index(Request $request, PaginatorInterface $paginator, $page_no = 1)
    {
        $qb = $this->ibn4204TopicRepository
            ->findBy(
                [],
                ['sort_no' => 'DESC']
            );

        $pagination = $paginator->paginate(
            $qb,
            $page_no,
            $this->eccubeConfig->get('eccube_default_page_count')
        );

        return [
            'pagination' => $pagination,
        ];
    }

    /**
     * 登録・編集する。
     *
     * @Route("/%eccube_admin_route%/content/ibn4204_topic/new", name="admin_content_ibn4204_topic_new")
     * @Route("/%eccube_admin_route%/content/ibn4204_topic/{id}/edit", requirements={"id" = "\d+"}, name="admin_content_ibn4204_topic_edit")
     * @Template("@ForIbn4204/admin/ibn4204_topic_edit.twig")
     *
     * @param Request $request
     * @param null $id
     *
     * @return array|\Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function edit(Request $request, Ibn4204Topic $Ibn4204Topic = null, CacheUtil $cacheUtil)
    {
        if (is_null($Ibn4204Topic)) {
            $Ibn4204Topic = $this->ibn4204TopicRepository->findOneBy([], ['sort_no' => 'DESC']);
            $sortNo = 1;
            if ($Ibn4204Topic) {
                $sortNo = $Ibn4204Topic->getSortNo() + 1;
            }

            $Ibn4204Topic = new \Plugin\ForIbn4204\Entity\Ibn4204Topic();
            $Ibn4204Topic
				->setTopicText3(new \DateTime())
                ->setSortNo($sortNo)
                ->setFixed(true)
                ->setVisible(true);
        }

        $builder = $this->formFactory
            ->createBuilder(Ibn4204TopicType::class, $Ibn4204Topic);

        $form = $builder->getForm();
		
        //▼画像アップロード
		// 既に画像保存されてる場合は取得する
        $oldTopicImage = $Ibn4204Topic->getTopicImage();
        $form->setData($Ibn4204Topic);
		//▲

        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {

			//▼画像アップロード
            $Ibn4204Topic = $form->getData();
            // ファイルアップロード
            $file = $form['topic_image']->getData();
            $fs = new Filesystem();
            if ($file && $fs->exists($this->getParameter('eccube_temp_image_dir').'/'.$file)) {
                $fs->rename(
                    $this->getParameter('eccube_temp_image_dir').'/'.$file,
                    $this->getParameter('eccube_save_image_dir').'/'.$file
                );
            }

            if (!$Ibn4204Topic->getUrl()) {
                $Ibn4204Topic->setLinkMethod(false);
            }
			//▲
			
            $this->ibn4204TopicRepository->save($Ibn4204Topic);

            $this->addSuccess('admin.common.save_complete', 'admin');

            // キャッシュの削除
            $cacheUtil->clearDoctrineCache();

            return $this->redirectToRoute('admin_content_ibn4204_topic_edit', ['id' => $Ibn4204Topic->getId()]);
        }

        return [
            'form' => $form->createView(),
            'Ibn4204Topic' => $Ibn4204Topic,
            'oldTopicImage' => $oldTopicImage, //▼画像アップロード
        ];
    }

    //▼画像アップロード
	/**
     * @Route("/%eccube_admin_route%/content/ibn4204_topic/image/process", name="admin_ibn4204_topic_image_process")
     */
    public function imageProcess(Request $request)
    {
        if (!$request->isXmlHttpRequest()) {
            throw new BadRequestHttpException();
        }

        $images = $request->files->get('admin_ibn4204_topic'); //Ibn4204TopicType.phpの最下部、return 'admin_ibn4204_topic'
        $allowExtensions = ['gif', 'jpg', 'jpeg', 'png'];
        $filename = null;
        if (isset($images['topic_image_file'])) {
            $image = $images['topic_image_file'];

            //ファイルフォーマット検証
            $mimeType = $image->getMimeType();
            if (0 !== strpos($mimeType, 'image')) {
                throw new UnsupportedMediaTypeHttpException();
            }

            // 拡張子
            $extension = $image->getClientOriginalExtension();
            if (!in_array($extension, $allowExtensions)) {
                throw new UnsupportedMediaTypeHttpException();
            }

            $filename = date('mdHis').uniqid('_').'.'.$extension;
            $image->move($this->getParameter('eccube_temp_image_dir'), $filename);
        }

        return new Response($filename);
    }

	// ▼画像アップロード
    /**
     * アップロード画像を取得する際にコールされるメソッド.
     *
     * @see https://pqina.nl/filepond/docs/api/server/#load
     * @Route("/%eccube_admin_route%/content/ibn4204_topic/image/load", name="admin_ibn4204_topic_image_load")
     */
    public function imageLoad(Request $request)
    {
        if (!$request->isXmlHttpRequest()) {
            throw new BadRequestHttpException();
        }

        $dirs = [
            $this->eccubeConfig['eccube_save_image_dir'],
            $this->eccubeConfig['eccube_temp_image_dir'],
        ];

        foreach ($dirs as $dir) {
            $image = \realpath($dir.'/'.$request->query->get('source'));
            $dir = \realpath($dir);

            if (\is_file($image) && \str_starts_with($image, $dir)) {
                $file = new \SplFileObject($image);

                return $this->file($file, $file->getBasename());
            }
        }

        throw new NotFoundHttpException();
    }

	// ▼画像アップロード
    /**
     * アップロード画像をすぐ削除する際にコールされるメソッド.
     *
     * @see https://pqina.nl/filepond/docs/api/server/#revert
     * @Route("/%eccube_admin_route%/content/ibn4204_topic/image/revert", name="admin_ibn4204_topic_image_revert")
     */
    public function imageRevert(Request $request)
    {
        if (!$request->isXmlHttpRequest() && $this->isTokenValid()) {
            throw new BadRequestHttpException();
        }

        $tempFile = $this->eccubeConfig['eccube_temp_image_dir'].'/'.$request->getContent();
        if (is_file($tempFile) && stripos(realpath($tempFile), $this->eccubeConfig['eccube_temp_image_dir']) === 0) {
            $fs = new Filesystem();
            $fs->remove($tempFile);

            return new Response(null, Response::HTTP_NO_CONTENT);
        }

        throw new NotFoundHttpException();
    }

    /**
     * 指定したデータを削除する。
     *
     * @Route("/%eccube_admin_route%/content/ibn4204_topic/{id}/delete", requirements={"id" = "\d+"}, name="admin_content_ibn4204_topic_delete", methods={"DELETE"})
     *
     * @param Request $request
     * @param ListpageImgcont $ListpageImgcont
     *
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function delete(Request $request, Ibn4204Topic $Ibn4204Topic, CacheUtil $cacheUtil)
    {
        $this->isTokenValid();

        log_info('トピック削除開始', [$Ibn4204Topic->getId()]);

        try {
            $this->ibn4204TopicRepository->delete($Ibn4204Topic);
            $this->addSuccess('admin.common.delete_complete', 'admin');

            log_info('トピック削除完了', [$Ibn4204Topic->getId()]);

            // キャッシュの削除
            $cacheUtil->clearDoctrineCache();

        } catch (\Exception $e) {
            $message = trans('admin.common.delete_error_foreign_key', ['%name%' => $Ibn4204Topic->getTopImage()]);
            $this->addError($message, 'admin');

            log_error('トピック削除エラー', [$Ibn4204Topic->getId(), $e]);
        }

        $sortNo = 1;
        $Ibn4204Topics = $this->ibn4204TopicRepository
            ->findBy([], ['sort_no' => 'ASC']);

        foreach ($Ibn4204Topics as $Ibn4204Topic) {
                $Ibn4204Topic->setSortNo($sortNo);
                $sortNo++;
        }
        $this->entityManager->flush();

        return $this->redirectToRoute('admin_content_ibn4204_topic');
    }

    /**
     * @Route("/%eccube_admin_route%/content/ibn4204_topic/{id}/visibility", requirements={"id" = "\d+"}, name="admin_content_ibn4204_topic_visibility", methods={"PUT"})
     */
    public function visibility(Request $request, Ibn4204Topic $Ibn4204Topic)
    {
        $this->isTokenValid();

        // 表示・非表示を切り替える
        if ($Ibn4204Topic->isVisible()) {
            $message = trans('admin.common.to_hide_complete', ['%name%' => $Ibn4204Topic->getId()]);
            $Ibn4204Topic->setVisible(false);
        } else {
            $message = trans('admin.common.to_show_complete', ['%name%' => $Ibn4204Topic->getId()]);
            $Ibn4204Topic->setVisible(true);
        }
        $this->entityManager->persist($Ibn4204Topic);

        $this->entityManager->flush();

        $this->addSuccess($message, 'admin');

        return $this->redirectToRoute('admin_content_ibn4204_topic');
    }

    /**
     * @Route("/%eccube_admin_route%/content/ibn4204_topic/sort_no/move", name="admin_content_ibn4204_topic_sort_no_move", methods={"POST"})
     *
     * @param Request $request
     *
     * @return Response
     */
    public function moveSortNo(Request $request)
    {
        if (!$request->isXmlHttpRequest()) {
            throw new BadRequestHttpException();
        }

        if ($this->isTokenValid()) {
            $sortNos = $request->request->all();
            foreach ($sortNos as $ibn4204TopicId => $sortNo) {
                $Ibn4204Topic = $this->ibn4204TopicRepository->find($ibn4204TopicId);
                $Ibn4204Topic->setSortNo($sortNo);
                $this->entityManager->persist($Ibn4204Topic);
            }
            $this->entityManager->flush();
        }

        return $this->json('OK', 200);
    }

}
