<?php

/*
 * This file is part of EC-CUBE
 *
 * Copyright(c) LOCKON CO.,LTD. All Rights Reserved.
 *
 * http://www.lockon.co.jp/
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Plugin\ForIbn4204\Form\Type;

use Eccube\Common\EccubeConfig;
use Plugin\ForIbn4204\Entity\Ibn4204Eyecatch;
use Eccube\Form\Type\ToggleSwitchType;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\CheckboxType;
use Symfony\Component\Form\Extension\Core\Type\FileType;
use Symfony\Component\Form\Extension\Core\Type\HiddenType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\TextareaType;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\Extension\Core\Type\DateTimeType;
use Symfony\Component\Form\Extension\Core\Type\IntegerType;
use Symfony\Component\Form\Extension\Core\Type\NumberType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\FormError;
use Symfony\Component\Form\FormEvents;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\Validator\Constraints as Assert;

class Ibn4204EyecatchType extends AbstractType
{
    /**
     * @var EccubeConfig
     */
    protected $eccubeConfig;

    public function __construct(EccubeConfig $eccubeConfig)
    {
        $this->eccubeConfig = $eccubeConfig;
    }

    /**
     * {@inheritdoc}
     */
    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $builder
			->add('check01', ChoiceType::class, [
                'choices' => [
                    '画像' => '1',
                    'MP4' => '2',
                    'YouTube' => '3',
                    'その他' => '4',
                ],
                'expanded' => true,
                'multiple' => false,
            ])
            ->add('doga', TextType::class, [
                'required' => false,
            ])
            ->add('sonota', TextareaType::class, [
                'required' => false,
                'attr' => [
                    'rows' => 8,
                ],
            ])
            ->add('image01_file', FileType::class, [
                'mapped' => false,
                'required' => false,
            ])
            ->add('image01', TextType::class, [
                'required' => false,
            ])
            ->add('image02_file', FileType::class, [
                'mapped' => false,
                'required' => false,
            ])
            ->add('image02', TextType::class, [
                'required' => false,
            ])
            ->add('image03_file', FileType::class, [
                'mapped' => false,
                'required' => false,
            ])
            ->add('image03', TextType::class, [
                'required' => false,
            ])
            ->add('image04_file', FileType::class, [
                'mapped' => false,
                'required' => false,
            ])
            ->add('image04', TextType::class, [
                'required' => false,
            ])
            ->add('image05_file', FileType::class, [
                'mapped' => false,
                'required' => false,
            ])
            ->add('image05', TextType::class, [
                'required' => false,
            ])
            ->add('image06_file', FileType::class, [
                'mapped' => false,
                'required' => false,
            ])
            ->add('image06', TextType::class, [
                'required' => false,
            ])
            ->add('link01', TextType::class, [
                'required' => false,
            ])
            ->add('link02', TextType::class, [
                'required' => false,
            ])
            ->add('link03', TextType::class, [
                'required' => false,
            ])
            ->add('link04', TextType::class, [
                'required' => false,
            ])
            ->add('link05', TextType::class, [
                'required' => false,
            ])
            ->add('link06', TextType::class, [
                'required' => false,
            ])
            ->add('window01', CheckboxType::class, [
                'required' => false,
                'label' => '別ウィンドウで開く',
                'value' => '1',
            ])
            ->add('window02', CheckboxType::class, [
                'required' => false,
                'label' => '別ウィンドウで開く',
                'value' => '1',
            ])
            ->add('window03', CheckboxType::class, [
                'required' => false,
                'label' => '別ウィンドウで開く',
                'value' => '1',
            ])
            ->add('window04', CheckboxType::class, [
                'required' => false,
                'label' => '別ウィンドウで開く',
                'value' => '1',
            ])
            ->add('window05', CheckboxType::class, [
                'required' => false,
                'label' => '別ウィンドウで開く',
                'value' => '1',
            ])
            ->add('window06', CheckboxType::class, [
                'required' => false,
                'label' => '別ウィンドウで開く',
                'value' => '1',
            ])
            ->add('title01', TextType::class, [
                'required' => false,
            ])
            ->add('title02', TextType::class, [
                'required' => false,
            ])
            ->add('title03', TextType::class, [
                'required' => false,
            ])
            ->add('title04', TextType::class, [
                'required' => false,
            ])
            ->add('title05', TextType::class, [
                'required' => false,
            ])
            ->add('title06', TextType::class, [
                'required' => false,
            ])
            ->add('comment01', TextareaType::class, [
                'required' => false,
                'attr' => [
                    'rows' => 3,
                ],
            ])
            ->add('comment02', TextareaType::class, [
                'required' => false,
                'attr' => [
                    'rows' => 3,
                ],
            ])
            ->add('comment03', TextareaType::class, [
                'required' => false,
                'attr' => [
                    'rows' => 3,
                ],
            ])
            ->add('comment04', TextareaType::class, [
                'required' => false,
                'attr' => [
                    'rows' => 3,
                ],
            ])
            ->add('comment05', TextareaType::class, [
                'required' => false,
                'attr' => [
                    'rows' => 3,
                ],
            ])
            ->add('comment06', TextareaType::class, [
                'required' => false,
                'attr' => [
                    'rows' => 3,
                ],
            ])
            ->add('fixed', HiddenType::class)
            ->add('visible', ChoiceType::class, [
                'label' => false,
                'choices' => ['admin.content.news.display_status__show' => true, 'admin.content.news.display_status__hide' => false],
                'required' => true,
                'expanded' => false,
            ])
			;
    }

    /**
     * {@inheritdoc}
     */
    public function configureOptions(OptionsResolver $resolver)
    {
        $resolver->setDefaults([
            'data_class' => 'Plugin\ForIbn4204\Entity\Ibn4204Eyecatch',
        ]);
    }

    /**
     * {@inheritdoc}
     */
    public function getBlockPrefix()
    {
        return 'admin_ibn4204_eyecatch';
    }
}
